#include <iom8v.h>
#include <macros.h>
#include <stdlib.h>
#include "SPI_MAX7221.h"
#include "TWI_DS1307.h"
#include "var.h"
#include "in_out.h"
#include "eeprom.h"

void uart0_init(void);
void re2rx(void);
void rs232_handler(void);
unsigned char rx2ds(unsigned char ind_mas);

//UART0 initialize
// desired baud rate: 37500
// actual: baud rate:37500 (0,0%)
// char size: 8 bit
// parity: Even
void uart0_init(void)
{
 UCSRB = 0x00; //disable while setting baud rate
 UCSRA = 0x00;
 UCSRC = BIT(URSEL) | 0x26;
 UBRRL = 0x09; //set baud rate lo
 UBRRH = 0x00; //set baud rate hi
 UCSRB = 0x90;// +,  +
}

#pragma interrupt_handler uart0_rx_isr:12
void uart0_rx_isr(void)
{
 //uart has received a character in UDR
 	rx_mas[rx_s] = UDR;
	rx_s++;
	rs232_count = 0;
	f_rs232 |= 0x02;
}

#pragma interrupt_handler uart0_tx_isr:14
void uart0_tx_isr(void)
{
 //character has been transmitted
 	if(tx_count == tx_s)
	{
		UCSRB = 0x90;// +,  +
  		tx_s = 0;
  		tx_count = 0;
 	}
 	else
	{
		UDR = tx_mas[tx_count];
		tx_count++;
 	}
}

void re2rx(void)
{
 	UCSRB = 0x90;// +,  +
 	rx_s = 0;
 	rx_mas[rx_s] = UDR;
 	tx_s = 0;
 	f_rs232 &= 0xFC;//  
}

void rs232_handler(void)
{
 	unsigned char temp_i = 0, temp_j = 0;
	if(f_rs232&0x01)//  
	{
	 	if((rx_s < 3)||(rx_s > 102)||(rx_mas[0] != 0x32)||(rx_mas[1] != rx_s))
		{
		 	re2rx();
			return;
		}
		wr_int2max(0x0A, 0x00);// 
		TCCR1B = 0x11;// 
		OCR1AH = 0x00;
 		OCR1AL = 0xF0;
		if(rx_mas[2] == 0x00)// 
		{
			tx_s = 3;
			for(temp_i = 1; temp_i < rx_s; temp_i++) tx_mas[temp_i] = rx_mas[temp_i];
		}
		if(rx_mas[2] == 0x01)// 
		{
			WR_DS1307(addr_hours, rx2ds(3));
			WR_DS1307(addr_mins, rx2ds(4));
			WR_DS1307(addr_secs, rx2ds(5));
			WR_DS1307(addr_day, rx2ds(6));
			WR_DS1307(addr_date, rx2ds(7));
			WR_DS1307(addr_mons, rx2ds(8));
			WR_DS1307(addr_years, rx2ds(9));
			tx_s = 10;
			for(temp_i = 1; temp_i < rx_s; temp_i++) tx_mas[temp_i] = rx_mas[temp_i];
		}
		if(rx_mas[2] == 0x02)// 
		{
		 	WR_DS1307(addr_secs, 0);
			tx_s = 3;
			for(temp_i = 1; temp_i < rx_s; temp_i++) tx_mas[temp_i] = rx_mas[temp_i];
		}
		if(rx_mas[2] == 0x03)// 
		{
			for(temp_i = 0; temp_i < 16; temp_i++)
			{
			 	mas_type_alarm[temp_i] = rx_mas[3 + (temp_i * 5)];
				mas_hour_alarm[temp_i] = rx_mas[4 + (temp_i * 5)];
				mas_min_alarm[temp_i] = rx_mas[5 + (temp_i * 5)];
				mas_date_alarm[temp_i] = rx_mas[6 + (temp_i * 5)];
				mas_mon_alarm[temp_i] = rx_mas[7 + (temp_i * 5)];
				EEPROM_WRITE((int)&EE_type_alarm[temp_i], mas_type_alarm[temp_i]);
				EEPROM_WRITE((int)&EE_hour_alarm[temp_i], mas_hour_alarm[temp_i]);
				EEPROM_WRITE((int)&EE_min_alarm[temp_i], mas_min_alarm[temp_i]);
				EEPROM_WRITE((int)&EE_date_alarm[temp_i], mas_date_alarm[temp_i]);
				EEPROM_WRITE((int)&EE_mon_alarm[temp_i], mas_mon_alarm[temp_i]);
			}
			len_alarm = rx_mas[83];
			EEPROM_WRITE((int)&EE_len_alarm, len_alarm);
			tx_s = 84;
			for(temp_i = 1; temp_i < rx_s; temp_i++) tx_mas[temp_i] = rx_mas[temp_i];
		}
		if(rx_mas[2] == 0xA1)// 
		{
			tx_mas[3] = ((hour_1 * 10) + hour_2);
			tx_mas[4] = ((min_1 * 10) + min_2);
			tx_mas[5] = ((date_1 * 10) + date_2);
			tx_mas[6] = ((mon_1 * 10) + mon_2);
			tx_mas[7] = ((year_1 * 10) + year_2);
			tx_s = 8;
		}
		if(rx_mas[2] == 0xA3)// 
		{
			read_eeprom_data();
			for(temp_i = 0; temp_i < 16; temp_i++)
			{
				tx_mas[3 + (temp_i * 5)] = mas_type_alarm[temp_i];
				tx_mas[4 + (temp_i * 5)] = mas_hour_alarm[temp_i];
				tx_mas[5 + (temp_i * 5)] = mas_min_alarm[temp_i];
				tx_mas[6 + (temp_i * 5)] = mas_date_alarm[temp_i];
				tx_mas[7 + (temp_i * 5)] = mas_mon_alarm[temp_i];
			}
			tx_mas[83] = len_alarm;
			tx_s = 84;
		}
		tx_mas[0] = 0x45;
		rx_s = 0;
		UCSRB = 0x48;// +,  +
		tx_count = 1;
		UDR = tx_mas[0];
		f_rs232 &= 0xFC;//  
		rs232_count = 0;
		wr_int2max(0x0A, 0xFF);// 
		TCCR1B = 0x00;// 
	}
}

unsigned char rx2ds(unsigned char ind_mas)
{
	return (((rx_mas[ind_mas] / 10) << 4) | (rx_mas[ind_mas] - ((rx_mas[ind_mas] / 10) * 10)));
}
