//ICC-AVR application builder : 25.08.2009 11:12:49
// Target : M8
// Crystal: 6.0000Mhz

#include <iom8v.h>
#include <macros.h>
#include <stdlib.h>
#include "SPI_MAX7221.h"
#include "TWI_DS1307.h"
#include "var.h"
#include "in_out.h"
#include "photo.h"
#include "eeprom.h"
#include "RS232.h"
#include "TWI_LM76.h"

unsigned char music(unsigned char first, unsigned char last);

void port_init(void)
{
 PORTB = 0x01;
 DDRB  = 0x29|0x02;
 PORTC = 0x0E; //m103 output only
 DDRC  = 0x00;
 PORTD = 0xE4;
 DDRD  = 0xC0;
}

//Watchdog initialize
// prescale: 2048K
void watchdog_init(void)
{
 	WDR(); //this prevents a timout on enabling
	WDTCR = 0x18;
	WDTCR = 0x0F; //WATCHDOG ENABLED - dont forget to issue WDRs
}

//SPI initialize
// clock rate: 921599hz
void spi_init(void)
{
 SPCR = 0x00; //setup SPI
 SPSR = 0x00; //setup SPI
}

#pragma interrupt_handler int0_isr:2
void int0_isr(void)
{
 	if(!(f_disp_off&0x01)) if(intensity > min_intensity) VDs(1);//   
	dots_count = 0;
	if(!(flags&0x02)) update_time();
	flags |= 0x01;//   /
}

//TIMER1 initialize - prescale:1
// WGM: 9) PWM phz freq correct, TOP=OCRnA
// desired value: 440Hz
// actual value: 440,012Hz (0,0%)
void timer1_init(void)
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0x00; //setup
 TCNT1L = 0x00;
 OCR1AH = 0x0F;
 OCR1AL = 0xA2;
 OCR1BH = 0x1A;
 OCR1BL = 0xA2;
 ICR1H  = 0xA2;
 ICR1L  = 0x1A;
 TCCR1A = 0x41;
 TCCR1B = 0x00;// 
 //0x11; //start Timer
}

//TIMER0 initialize - prescale:1024
// desired value: 40mSec
// actual value: 39,936mSec (0,2%)
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 TCNT0 = 0x16; //set count
 TCCR0 = 0x05; //start timer
}

#pragma interrupt_handler timer0_ovf_isr:10
void timer0_ovf_isr(void)
{
 TCNT0 = 0x16; //reload counter value
 dots_count += d_main_count;
 keys_count += d_main_count;
 if(f_alarm_on&0x01)
 {
  	alarm_count += d_main_count;
 	if(alarm_count > (((unsigned long)(len_alarm)) << 20)) f_alarm_on &= 0xFE;
 }
 if(f_mus&0x01) mus_count += d_main_count;
 if(f_rs232&0x02)
 {
  	rs232_count += d_main_count;
	if(rs232_count > d_rs232)
	{
	 	f_rs232 |= 0x01;
		f_rs232 &= 0xFD;
		UCSRB = 0x48;// +,  +
	}
 }
}

//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 port_init();
 watchdog_init();
 spi_init();
 uart0_init();
 timer0_init();
 timer1_init();

 MCUCR = 0x02;//    
 GICR  = 0x40;//   int0
 TIMSK = 0x01; //timer interrupt sources
 SEI(); //re-enable interrupts
 init_display();
 //all peripherals are now initialized
}

unsigned char temp_main_char = 0, temp_main_char_2 = 0, temp_main_char_3 = 0, temp_main_char_4 = 0;

void main()
{
 	init_devices();
	read_eeprom_data();
	
	wr_int2max(0x09, 0x00);// .
	wr_int2max(0x01, 0x7E);//1 = O
	wr_int2max(0x02, 0x05);//2 = r
	wr_int2max(0x03, 0x7E);//3 = O
	wr_int2max(0x04, 0x0E);//4 = L
	wr_int2max(0x05, 0x7E);//5 = O
	wr_int2max(0x06, 0x5F);//6 = G
	wr_int2max(0x07, 0x30);//7 = I
	wr_int2max(0x08, 0x7E);//1 = O
	for(i2 = 0; i2 < 65000; i2++) WDR();
	init_display();
	
	WR_DS1307(control_reg_addr, control_reg_val);
	WR_DS1307(addr_secs, (RE_DS1307(addr_secs)&0x7F));
	WR_DS1307(addr_hours, (RE_DS1307(addr_hours)&0xBF));
	
	while(1)
	{
//-------------------------------- --------------------------------
		if(dots_count >= d_VDs)
		{
		 	VDs(0);// 
			dots_count = 0;
			if(!(f_disp_off&0x02)) set_intensity();//  
			else wr_int2max(0x0A, 0x00);//  = 0
			temperature = ((RE_TEMP()&0x7FFF) >> 3);
		}
//-------------------------------- --------------------------------
//-----------------------------   ---------------------------
		if((flags&0x03) == 0x01)
		{
		 	flags &= 0xFE;//   /
			if(view != 3) show_dig_1(hour_1, hour_2, min_1, min_2);
			if(view == 0) show_dig_2(date_1, date_2|0x80, mon_1, mon_2);
			if(view == 1) show_dig_2(0x0F, 0x0F, sec_1, sec_2);
			if(view == 2) show_dig_2(0x0F, 0x0F, (temperature / 10), ((temperature - ((temperature / 10) * 10))|0x80));
			if(view == 3)
			{
			 	show_dig_1(date_1, date_2, mon_1, mon_2);
				show_dig_2(0x02, 0x00, year_1, year_2);
			}
		}
		if(flags&0x02)
		{
		 	if(menu_st == set_hours) show_dig_1((hour_1|0x80), (hour_2|0x80), min_1, min_2);// 
			if(menu_st == set_mins) show_dig_1(hour_1, hour_2, (min_1|0x80), (min_2|0x80));// 
			if(menu_st == set_secs) show_dig_1(0x0A, 0x0A, sec_1|0x80, sec_2|0x80);// 
			if(menu_st == set_date) show_dig_1((date_1|0x80), (date_2|0x80), mon_1, mon_2);// 
			if(menu_st == set_day) show_dig_1(0x0F, 0x0F, 0x0F, day|0x80);//  
			if(menu_st == set_mons) show_dig_1(date_1, date_2, (mon_1|0x80), (mon_2|0x80));// 
			if(menu_st == set_years) show_dig_1(0x02, 0x00, year_1|0x80, year_2|0x80);// 
//------------      ------------
			al_num_1 = (num_alarm + 1) / 10;
			al_num_2 = ((num_alarm + 1) - (al_num_1 * 10));
			al_type_1 = type_alarm / 10;
			al_type_2 = (type_alarm - (al_type_1 * 10));
			al_hour_1 = al_hour / 10;
			al_hour_2 = (al_hour - (al_hour_1 * 10));
			al_min_1 = al_min / 10;
			al_min_2 = (al_min - (al_min_1 * 10));
			al_date_1 = al_date / 10;
			al_date_2 = (al_date - (al_date_1 * 10));
			al_mon_1 = al_mon / 10;
			al_mon_2 = (al_mon - (al_mon_1 * 10));
//------------      ------------
			if(menu_st == set_alarm_num) show_dig_1(0x0C, 0x0A, al_num_1|0x80, al_num_2|0x80);//  
			if(menu_st == set_alarm_type) show_dig_1(0x0F, 0x0F, 0x0F, al_type_2|0x80);//  
			if(menu_st == set_alarm_hours) show_dig_1((al_hour_1|0x80), (al_hour_2|0x80), al_min_1, al_min_2);//  
			if(menu_st == set_alarm_mins) show_dig_1(al_hour_1, al_hour_2, (al_min_1|0x80), (al_min_2|0x80));//  
			if(menu_st == set_alarm_date) show_dig_1((al_date_1|0x80), (al_date_2|0x80), al_mon_1, al_mon_2);//  
			if(menu_st == set_alarm_mons) show_dig_1(al_date_1, al_date_2, (al_mon_1|0x80), (al_mon_2|0x80));//  
			i = menu_st / 10;
			j = (menu_st - (i * 10));
			show_dig_2(0x05, 0x0A, i, j);
		}
//-----------------------------   ---------------------------
//-------------------------------- ------------------------------
		if((!(f_alarm_on&0x01))&&(!(flags&0x02)))
		{
			for(i = 0; i < 16; i++)
			{
			 	if(mas_type_alarm[i] != alarm_type_off)
				{
				 	j = ((hour_1 * 10) + hour_2);
					k = ((min_1 * 10) + min_2);
					l = ((date_1 * 10) + date_2);
					m = ((mon_1 * 10) + mon_2);
					if((mas_hour_alarm[i] == j)&&(mas_min_alarm[i] == k))
					{
					 	if((mas_type_alarm[i] == alarm_type_ever)||
						((mas_type_alarm[i] == alarm_type_mor)&&(day != 0x06)&&(day != 0x07))||
						((mas_type_alarm[i] == alarm_type_dayoff)&&((day == 0x06)||(day == 0x07)))||
						((mas_type_alarm[i] == alarm_type_sin)&&(mas_date_alarm[i] == l)&&(mas_mon_alarm[i] == m)))
							f_alarm_on |= 0x01;
						if((sec_1 > 0)||(sec_2 > 5)||(sec_2 < 1)) f_alarm_on &= 0xFE;
					}
				}
				alarm_count = 0;
				while(f_alarm_on&0x01)
				{
					show_dig_1(0x0A, 0x0A, 0x0A, 0x0A);
					show_dig_2(0x0A, 0x0A, 0x0A, 0x0A);
					if((music(0, 32))||(music(0, 59))||
					(music(32, 111))||(music(60, 121)))
					{
					 	TCCR1B = 0x00;// 
						break;
					}
					TCCR1B = 0x00;// 
				}
			}
		}
//-------------------------------- ------------------------------
		sets();// /, , 
		rs232_handler();//  
	 	WDR();
	}
}

unsigned char music(unsigned char first, unsigned char last)
{
 	unsigned char temp_j = 0, temp_k = 0;
	unsigned int temp_i = 0;
	for(temp_j = first; temp_j < last; temp_j++)
	{
		//if((first == 0)&&(temp_j == 62)) temp_j++;
		if((first == 0)&&(temp_j == 30)&&(last > 32)) temp_j++;
		if((first == 60)&&(temp_j == 60)&&(last == 121)) temp_j++;
		if(keys()||(!(f_alarm_on&0x01)))
		{
			f_alarm_on &= 0xFE;
			return 1;
		}
		EEPROM_READ((int)&EE_mus_note[temp_j], temp_k);
		temp_i = note_mas[temp_k];
		TCCR1B = 0x00;// 
		
		EEPROM_READ((int)&EE_mus_long[temp_j], temp_k);
		
		if((first == 0)&&(temp_j == 31)&&(last > 32)) temp_k = 0x1F;
		if((first == 32)&&(temp_j == 110)&&(last == 111)) temp_k = 0x1D;
		if((first == 60)&&(temp_j == 61)&&(last == 121)) temp_k = 0xEC;
		
		f_mus |= 0x01;
		mus_count = 0;
		while(mus_count < long_note_mas[temp_k >> 4])
		{
			if(keys()||(!(f_alarm_on&0x01)))
			{
				f_alarm_on &= 0xFE;
				return 1;
			}
			WDR();
		}
		f_mus &= 0xFE;
		
		
		TCCR1B = 0x11;// 
		OCR1AH = (temp_i >> 8);
 		OCR1AL = (temp_i&0x00FF);
		
		
		f_mus |= 0x01;
		mus_count = 0;
		while(mus_count < long_note_mas[temp_k&0x0F])
		{
			if(keys()||(!(f_alarm_on&0x01)))
			{
				f_alarm_on &= 0xFE;
				return 1;
			}
			WDR();
		}
	}
	if(!(f_alarm_on&0x01))
	{
		TCCR1B = 0x00;// 
		return 1;
	}
	return 0;
}